# -*- coding: utf-8 -*-

# Copyright (c) 2002 - 2007 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog to enter options used to start a project in the VCS.
"""

import os

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from KdeQt import KQFileDialog

from E4Gui.E4Completers import E4DirCompleter

from Ui_CvsOptionsDialog import Ui_CvsOptionsDialog

import Utilities

class CvsOptionsDialog(QDialog, Ui_CvsOptionsDialog):
    """
    Class implementing a dialog to enter options used to start a project in 
    the repository.
    """
    def __init__(self, project, archiveName, archiveEditable = False, parent = None):
        """
        Constructor
        
        @param project reference to the project object
        @param archiveName name of the archive (string or QString)
        @param archiveEditable flag indicating that the archives name may be changed
                (boolean)
        @param parent parent widget (QWidget)
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        
        self.vcsDirectoryCompleter = E4DirCompleter(self.vcsDirectoryEdit)
        
        self.project = project
        
        hd = Utilities.toNativeSeparators(QDir.homePath())
        hd = os.path.join(unicode(hd), 'cvsroot')
        self.vcsDirectoryEdit.setText(hd)
        
        self.vcsArchiveEdit.setText(archiveName)
        self.vcsArchiveEdit.setReadOnly(not archiveEditable)
        
        self.vcsVendorEdit.setText(archiveName)
        
        self.cvsTagValidator = QRegExpValidator(QRegExp(r"[a-zA-Z][a-zA-Z0-9_-]*"), self)
        self.vcsVendorEdit.setValidator(self.cvsTagValidator)
        self.vcsReleaseEdit.setValidator(self.cvsTagValidator)
        
    @pyqtSignature("")
    def on_vcsDirButton_clicked(self):
        """
        Private slot to display a directory selection dialog.
        """
        directory = KQFileDialog.getExistingDirectory(\
            self,
            self.trUtf8("Select Repository Directory"),
            self.vcsDirectoryEdit.text(),
            QFileDialog.Options(QFileDialog.ShowDirsOnly))
        
        if not directory.isEmpty():
            self.vcsDirectoryEdit.setText(Utilities.toNativeSeparators(directory))
        
    def getData(self):
        """
        Public slot to retrieve the data entered into the dialog.
        
        @return a dictionary containing the data entered
        """
        vcsdatadict = {
            "repository" : unicode(self.vcsDirectoryEdit.text()),
            "archive" : unicode(self.vcsArchiveEdit.text()),
            "vendor" : unicode(self.vcsVendorEdit.text()),
            "message" : unicode(self.vcsLogEdit.text()),
            "release" : unicode(self.vcsReleaseEdit.text()),
        }
        return vcsdatadict
